//
//  DBManager.swift
//  aufgabe_14_b
//
//  Created by Daniel Hoover on 07.12.16.
//  Copyright © 2016 Daniel Hoover. All rights reserved.
//

import UIKit

class DBManager: NSObject {
    
    var dbFileNameWithPathUrl : URL;
    var db: OpaquePointer? = nil //OpaquePointer is a wrapper for a C Type Pointer to structures that can't be represented in swift
    
    init(databaseFileName: String) {
        
        self.dbFileNameWithPathUrl = try! FileManager.default.url(for: .documentDirectory, in: .userDomainMask, appropriateFor: nil, create: false).appendingPathComponent(databaseFileName);
        
        super.init()
        
        copyDBFileToDocumentsIfNecessary(databaseFileName: databaseFileName)
        
        //beware - DO NOT USE sqlite3_open if you are not 100% sure that the db file that you want to open exists
        //because if it doesn't an empty database will be created and no error will be thrown
        //sqlite3_open_v2 helps because you can prevent that an empty database will be created
        if sqlite3_open_v2(dbFileNameWithPathUrl.path, &db, SQLITE_OPEN_READWRITE, nil) != SQLITE_OK {
            print("error opening database " + databaseFileName)
        }
    }
    
    
    func copyDBFileToDocumentsIfNecessary(databaseFileName: String) {
        
        if !FileManager.default.fileExists(atPath: dbFileNameWithPathUrl.path) { //if it doesn't exist
            
            var parts = databaseFileName.components(separatedBy: ".");
            
            let sourceDatabaseFile = Bundle.main.url(forResource: parts[0], withExtension: parts[1])! //get file url from bundle
            
            do {
                try FileManager.default.copyItem(at: sourceDatabaseFile, to: dbFileNameWithPathUrl) //copy it
            } catch let error as NSError {
                print("There was an error when trying to copy: \(error)")
            }
        }
            
    }
    
    
    func doTheMagicForTask14b() {
        
        var statement: OpaquePointer? = nil //again a C Type Pointer
        
        //the most complex thing that follows is the sql statement - if you didn't follow lecture Datenbanksysteme_I then
        //it is hard to understand
        
        //we are going to execute a query that
        
        //gets CustomerId from following customers:
        //Michelle Brooks from New York
        //Fynn Zimmermann from Frankfurt
        //Lucas Mancini from Rome
        
        //the resultset should be joined with all Invoices (Purchases) Invoice Table has CustomerId
        //the resultset should be joined with all Invoice Lines (Products bought) Invoice Line Table has InvoiceId
        
        //there we need the trackId
        //the resultset should be joined with Track
        
        //there we need Name AS TrackName,albumId,MediaTypeId,GenreId
        
        //the result set should be joined with MediaType mt ON t.MediaTypeId=mt.MediaTypeId
        //there we need mt.Name AS MediaTypeName
        
        //the result set should be joined with Genre g ON t.GenreId=g.GenreId
        //there we need g.Name AS GenreName
        
        //the result set should be joined with Album al ON t.AlbumId=al.AlbumId
        
        //there we need al.Title AS AlbumName and ArtistId
        
        //the result should finally be joined with Artist a ON al.AristId=a.ArtistId
        //there we need a.Name AS ArtistName
        
        
        let theQuery : String = "SELECT " +
                                    "a.Name AS ArtistName, " +
                                    "t.Name AS TrackName," +
                                    "al.Title AS AlbumName," +
                                    "g.Name AS GenreName," +
                                    "mt.Name AS MediaTypeName " +
                                "FROM Customer c LEFT JOIN " +
                                    "Invoice i ON c.CustomerId=i.CustomerId LEFT JOIN " +
                                    "InvoiceLine il ON il.InvoiceId=i.InvoiceId LEFT JOIN " +
                                    "Track t ON t.TrackId=il.TrackId LEFT JOIN " +
                                    "MediaType mt ON t.MediaTypeId=mt.MediaTypeId LEFT JOIN " +
                                    "Genre g ON t.GenreId=g.GenreId LEFT JOIN " +
                                    "Album al ON t.AlbumId=al.AlbumId LEFT JOIN " +
                                    "Artist a ON al.ArtistId=a.ArtistId " +
                                "WHERE "
                                + "(c.FirstName='Michelle' AND c.LastName='Brooks' AND c.City='New York') " +
                                "OR (c.FirstName='Fynn' AND c.LastName='Zimmermann' AND c.City='Frankfurt') " +
                                "OR (c.FirstName='Lucas' AND c.LastName='Mancini' AND c.City='Rome') "
        
        var error = false; //we are positive...
        
        //lets prepare the statement
        if sqlite3_prepare_v2(self.db, theQuery, -1, &statement, nil) != SQLITE_OK {
            
            let errmsg = String(cString: sqlite3_errmsg(self.db))
            print("error preparing select: \(errmsg)")
            error = true
            
        }
        
        if !error {
            
            let column1 = 20;
            let column2 = 25;
            let column3 = 18;
            let column4 = 10;
            let column5 = 25;
            
            let numCharsOfLine = column1+column2+column3+column4+column5+(5*3)+1;
            
            
            var ArtistString,TrackString,AlbumString,GenreString,MediaTypeString:String;
            
            //now make a beautiful output for the console...
            print(String(format: "| %@ | %@ | %@ | %@ | %@ |", "Artist".trimOrFill(length: column1), "Track".trimOrFill(length: column2), "Album".trimOrFill(length: column3), "Genre".trimOrFill(length: column4), "Medientyp".trimOrFill(length: column5)))
            
            
            //get all result values
            while sqlite3_step(statement) == SQLITE_ROW {
                
                print(String(format: "%@", "".trimOrFill(length: numCharsOfLine, char: "=")))
                
                ArtistString = String(cString: UnsafePointer(sqlite3_column_text(statement, 0)))
                TrackString = String(cString: UnsafePointer(sqlite3_column_text(statement, 1)))
                AlbumString = String(cString: UnsafePointer(sqlite3_column_text(statement, 2)))
                GenreString = String(cString: UnsafePointer(sqlite3_column_text(statement, 3)))
                MediaTypeString = String(cString: UnsafePointer(sqlite3_column_text(statement, 4)))
                
                print(String(format: "| %@ | %@ | %@ | %@ | %@ |", ArtistString.trimOrFill(length: column1), TrackString.trimOrFill(length: column2), AlbumString.trimOrFill(length: column3), GenreString.trimOrFill(length: column4), MediaTypeString.trimOrFill(length: column5)))
            } 
            
            print(String(format: "%@", "".trimOrFill(length: numCharsOfLine, char: "=")))
            
            sqlite3_finalize(statement) //destroy prepared statement
            
            statement = nil //set it to nil
            
            sqlite3_close(self.db) //close database connection
            
            self.db = nil
        }
    
        
    }
    
}
